// File string.h for CC1B
#ifndef _STRING_H_
#define _STRING_H_
//include other libraries here
#include <system/memory.h>
#pragma library 1

/**
 * \file string.h
 * \brief Library for string support
 *
 * This library provides functions for strings located in ram.
 * \n
 */
 
/**
 * \brief Calculate the length of a string
 *
 * The strlen() function calculates the length of the string s, not including the terminating `\0' character.
 *
 * \n
 * \param s Address of string in ram.
 * \return The strlen() function returns the length of string s.
 * \n
 */
int strlen(char *s)
{
  char n = 0;
  while (*s) {
    s++;
    n++;
  }
  return (n);
}

/**
 * \brief Concatenate string to string
 *
 * The strcat() function appends the t string to the s string overwriting the `\0' character\n
 * at the end of s, and then adds a terminating `\0' character.  The strings may not overlap,\n
 * and the s string must have enough space for the result.
 *
 * \n
 * \param s Address of string in ram.
 * \param t Address of string in ram.
 * \return The strcat() function returns a pointer to the destination string s.
 * \n
 */
char *strcat(char *s, char *t)
{
  char *d = s + strlen(s);
  while (*t) {
    _MainTemp = *t;
	*d = _MainTemp;
	d++;
	t++;
  }
  *d = 0;
  return (s);
}

/**
 * \brief Locate character in string
 *
 * The strchr() function searches a string for the presence of a specific character.
 *
 * \n
 * \param s Address of string in ram.
 * \param c Character to locate.
 * \return The strchr() function returns a pointer to the matched character or NULL if the character is not found.
 * \n
 */
char *strchr(char *s, char c)
{
  while (*s) {
    if (*s == c) return (s);
    ++s;
  }
  return (0);
}

/**
 * \brief Compare two strings
 *
 * The strcmp() function compares the two strings s and t.\n
 * It returns an integer less than, equal to, or greater than zero if s is found,\n
 * respectively, to be less than, to match, or be greater than t.
 *
 * \n
 * \param s Address of string in ram.
 * \param t Address of string in ram.
 * \return The strcmp() function returns an integer less than, equal to, or greater than zero\n
 * if s is found, respectively, to be less than, to match, or be greater than t.
 * \n
 */
int strcmp(char *s, char *t)
{
  char result;
  _MainTemp = *t;
  while (*s == _MainTemp) {
    if (*s == 0) {
	  return (0);
	}
    ++s;
	++t;
	_MainTemp = *t;
  }
  result = *s - _MainTemp;
  return (result);
}

/**
 * \brief Copy string to string
 *
 * The strcpy() function copies the string pointed to by t (including the terminating `\0' character)\n
 * to the array pointed to by s. The strings may not overlap, and the destination string s must be\n
 * large enough to receive the copy.
 *
 * \n
 * \param s Address of string in ram.
 * \param t Address of string in ram.
 * \return The strcpy() function returns a pointer to the destination string s.
 * \n
 */
char *strcpy(char *s, char *t)
{
  char *d = s;
  //char temp;
  while (*t) {
    _MainTemp = *t;
	*d = _MainTemp;
	d++;
	t++;
  }
  *d = 0;
  return (s);
}

/**
 * \brief Concatenate string to string for specified number of characters
 *
 * The strncat() function appends up to the specified number of characters from the t string to the\n
 * s string overwriting the `\0' character at the end of s, and then adds a terminating `\0' character.\n
 * The strings may not overlap, and the s string must have enough space for the result.
 *
 * \n
 * \param s Address of string in ram.
 * \param t Address of string in ram.
 * \param len Number of characters to copy.
 * \return The strncat() function returns a pointer to the destination string s.
 * \n
 */
char *strncat(char *s, char *t, int len)
{
  char *d = s + strlen(s);
  char temp;
  while (len) {
    len--;
    if (*t) {
	  temp = *t;
	  *d = temp;
	  d++;
	  t++;
    }
	else break;
  }
  *d = 0;
  return (s);
}

/**
 * \brief Compare two strings for up to a specified number of bytes
 *
 * The strncmp() function compares the two strings s and t for the first (at most) n characters.\n
 * It returns an integer less than, equal to, or greater than zero if s is (or the first n bytes thereof)\n
 * found, respectively, to be less than, to match, or be greater than t.
 *
 * \n
 * \param s Address of string in ram.
 * \param t Address of string in ram.
 * \param len Number of characters to compare.
 * \return The strncmp() function returns an integer less than, equal to, or greater than zero\n
 * if s (or the first n bytes thereof) is found, respectively, to be less than, to match, or be greater than t.
 * \n
 */
int strncmp(char *s, char *t, int len)
{
  char temp;
  while (len) {
    len--;
    temp = *t;
    if (*s == temp) {
      if (*s == 0) return (0);
      ++s;
	  ++t;
    }
  }
  if (len) return (*s - *t);
  return (0);
}

/**
 * \brief Copy string to string for specified number of characters
 *
 * The strncpy() function copies up to the specified number of characters from the t string\n
 * (including the terminating `\0' character) to the array pointed to by s. The strings may not overlap,\n
 * and the destination string s must be large enough to receive the copy.
 *
 * \n
 * \param s Address of string in ram.
 * \param t Address of string in ram.
 * \param len Number of bytes to copy.
 * \return The strncpy() function returns a pointer to the destination string s.
 * \n
 */
char *strncpy(char *s, char *t, int len)
{
  char *d = s;
  char temp;
  while (len > 0) {
    len--;
	if (*t) {
      temp = *t;
	  *d = temp;
	  d++;
	  t++;
	  continue;
	}
    while (len > 0) {
	  len--;
	  *d = 0;
	  d++;
    }
	return (s);
  }
  *d = 0;
  return (s);
}

/**
 * \brief Locate character in string
 *
 * The strrchr() function searches a string for the presence of a specific character.
 *
 * \n
 * \param s Address of string in ram.
 * \param c Character to locate.
 * \return The strrchr() function returns a pointer to the last occurrence of the character c in the string s.
 * \n
 */
char *strrchr(char *s, char c)
{
  char *ptr = 0;
  while (*s) {
    if (*s == c) ptr = s;
    ++s;
  }
  return (ptr);
}

#pragma library 0
#endif
// End of file string.h
